/**
 * This class is used to track a performance counter which is number based.
 * The user has access to many properties which give statistics of different nature.
 *
 * The implementer can track two kinds of Performance Counter: time and count.
 * For time you can optionally call fetchNewFrame() to notify the start of a new frame to monitor, then call beginMonitoring() to start and endMonitoring() to record the lapsed time. endMonitoring takes a newFrame parameter for you to specify if the monitored time should be set for a new frame or accumulated to the current frame being monitored.
 * For count you first have to call fetchNewFrame() to notify the start of a new frame to monitor, then call addCount() how many time required to increment the count value you monitor.
 */
export declare class PerfCounter {
    /**
     * Gets or sets a global boolean to turn on and off all the counters
     */
    static Enabled: boolean;
    /**
     * Returns the smallest value ever
     */
    get min(): number;
    /**
     * Returns the biggest value ever
     */
    get max(): number;
    /**
     * Returns the average value since the performance counter is running
     */
    get average(): number;
    /**
     * Returns the average value of the last second the counter was monitored
     */
    get lastSecAverage(): number;
    /**
     * Returns the current value
     */
    get current(): number;
    /**
     * Gets the accumulated total
     */
    get total(): number;
    /**
     * Gets the total value count
     */
    get count(): number;
    /**
     * Creates a new counter
     */
    constructor();
    /**
     * Call this method to start monitoring a new frame.
     * This scenario is typically used when you accumulate monitoring time many times for a single frame, you call this method at the start of the frame, then beginMonitoring to start recording and endMonitoring(false) to accumulated the recorded time to the PerfCounter or addCount() to accumulate a monitored count.
     */
    fetchNewFrame(): void;
    /**
     * Call this method to monitor a count of something (e.g. mesh drawn in viewport count)
     * @param newCount the count value to add to the monitored count
     * @param fetchResult true when it's the last time in the frame you add to the counter and you wish to update the statistics properties (min/max/average), false if you only want to update statistics.
     */
    addCount(newCount: number, fetchResult: boolean): void;
    /**
     * Start monitoring this performance counter
     */
    beginMonitoring(): void;
    /**
     * Compute the time lapsed since the previous beginMonitoring() call.
     * @param newFrame true by default to fetch the result and monitor a new frame, if false the time monitored will be added to the current frame counter
     */
    endMonitoring(newFrame?: boolean): void;
    /**
     * Call this method to end the monitoring of a frame.
     * This scenario is typically used when you accumulate monitoring time many times for a single frame, you call this method at the end of the frame, after beginMonitoring to start recording and endMonitoring(false) to accumulated the recorded time to the PerfCounter or addCount() to accumulate a monitored count.
     */
    endFrame(): void;
    /** @internal */
    _fetchResult(): void;
    private _startMonitoringTime;
    private _min;
    private _max;
    private _average;
    private _current;
    private _totalValueCount;
    private _totalAccumulated;
    private _lastSecAverage;
    private _lastSecAccumulated;
    private _lastSecTime;
    private _lastSecValueCount;
}
